@@ -10,7 +10,7 @@ local files and your environment.
 
 # type annotations
 from __future__ import annotations
-from typing import IO, Dict, TypeVar, Callable, Union, Iterable, Optional, Any
+from typing import IO, Tuple, List, Dict, NamedTuple, TypeVar, Callable, Union, Iterable, Optional, Any
 
 # standard libs
 import os
@@ -260,6 +260,28 @@ class Namespace(NSCoreMixin):
         """Translate namespace to an :class:`Environ` namespace."""
         return Environ(defaults=self)
 
+    def whereis(self, leaf: str, value: Union[Callable[[T], bool], T] = lambda _: True) -> List[Tuple[str, ...]]:
+        """
+        Find paths to `leaf`, optionally filtered on `value`.
+
+        Example:
+            >>> ns = Namespace({'a': {'x': 1, 'y': 2}, 'b': {'x': 3, 'z': 4}})
+            >>> ns
+            Namespace({'a': {'x': 1, 'y': 2}, 'b': {'x': 3, 'z': 4}})
+
+            >>> ns.whereis('x')
+            [('a',), ('b',)]
+
+            >>> ns.whereis('x', 1)
+            [('a',)]
+
+            >>> ns.whereis('x', lambda v: v % 3 == 0)
+            [('b',)]
+        """
+        check = value if callable(value) else lambda x: x == value
+        return [tuple(branch.stem[:-1]) for branch in _find_the_leaves(self) 
+                if branch.stem[-1] == leaf and check(branch.leaf)]
+
 
 # basic types automatically converted from environment variable
 _VT = TypeVar('_VT', str, int, float, bool, type(None))
@@ -310,6 +332,41 @@ def _flatten(ns: dict, prefix: str = None) -> dict:
         return {f'{prefix}_{key}': value for key, value in new.items()}
 
 
+class _Leaf(NamedTuple):
+    """Definition of a tree leaf."""
+    leaf: Any
+    stem: list[str]
+
+
+def _find_the_leaves(tree: Optional[Mapping[str, Any]]) -> List[_Leaf]:
+    """Return the leaves (and their stems) of the tree (e.g., Namespace)."""
+    leaves = []
+    if tree is not None:
+        leaves = [_Leaf(_read_a_leaf(stem, tree), stem) for stem in _walk_the_tree(tree)]
+    return leaves
+
+
+def _read_a_leaf(stem: List[str], tree: Mapping[str, Any]) -> Optional[Any]:
+    """Read the leaf at the end of the stem on the tree (e.g., Namespace)."""
+    try:
+        return reduce(lambda branch, leaf: branch[leaf], stem, tree)
+    except KeyError:
+        return None
+
+
+def _walk_the_tree(tree: Mapping[str, Any], stem: List[str] = None) -> List[List[str]]:
+    """Return the leaves of the branches."""
+    stem = stem or []
+    leaves = []
+    for branch, branches in tree.items():
+        leaf = stem + [branch, ]
+        if isinstance(branches, dict):
+            leaves.extend(_walk_the_tree(branches, leaf))
+        else:
+            leaves.append(leaf)
+    return leaves
+
+
 class Environ(NSCoreMixin):
     """
     A Namespace initialized via :func:`~Namespace.from_env`.
@@ -517,6 +574,27 @@ class Configuration(NSCoreMixin):
         else:
             raise KeyError(f'Not found: {path}')
 
+    def whereis(self, leaf: str,
+                value: Union[Callable[[T], bool], T] = lambda _: True) -> Dict[str, List[Tuple[str, ...]]]:
+        """
+        Find paths to `leaf`, optionally filtered on `value`, for each member namespace.
+
+        Example:
+            >>> one = Namespace({'a': {'x': 1, 'y': 2}, 'b': {'x': 3, 'z': 4}})
+            >>> two = Namespace({'b': {'x': 4}, 'c': {'j': True, 'k': 3.14}})
+            >>> cfg = Configuration(one=one, two=two)
+
+            >>> cfg.whereis('x')
+            {'one': [('a',), ('b',)], 'two': [('b',)]}
+
+            >>> cfg.whereis('x', 1)
+            {'one': [('a',)], 'two': []}
+
+            >>> cfg.whereis('x', lambda v: v % 3 == 0)
+            {'one': [('b',)], 'two': []}
+        """
+        return {name: space.whereis(leaf, value) for name, space in self.namespaces.items()}
+
     def __setattr__(self, name: str, value: Any) -> None:
         """Intercept parameter assignment."""
         if name in self:
